#version 330

uniform sampler2D tex;
uniform int radius = 5;
uniform int axis = 0;

out vec4 f_color;
in vec2 uv;

const float pi = 3.14159;

float gauss(float x, float sigma) {
    return 1.0f / (2.0f * pi * sigma * sigma) * exp(-(x * x) / (2.0f * sigma * sigma));
}

void main() {
  ivec2 tex_size = textureSize(tex, 0);
  vec2 texel_size = vec2(1.0 / tex_size.x, 1.0 / tex_size.y);
  int diameter = radius * 2 + 1;
  float sigma = radius * 0.7;

  float weight_sum = 0;
  vec3 total_color = vec3(0.0);
  if (axis == 0) {
    for (int i = 0; i < diameter; i++) {
        float w = gauss(i - radius, sigma);
        float sample_pos = uv.x + (i - radius) * texel_size.x;
        total_color += texture(tex, vec2(sample_pos, 1.0 - uv.y)).rgb * w * step(0.0, sample_pos) * step(sample_pos, 1.0);
        weight_sum += w;
    }
  }
  if (axis == 1) {
    for (int i = 0; i < diameter; i++) {
        float w = gauss(i - radius, sigma);
        float sample_pos = 1.0 - (uv.y + (i - radius) * texel_size.y);
        total_color += texture(tex, vec2(uv.x, sample_pos)).rgb * w * step(0.0, sample_pos) * step(sample_pos, 1.0);
        weight_sum += w;
    }
  }

  f_color = vec4(total_color / weight_sum, 1.0);
}