#version 330

uniform sampler2D map;

in vec2 vert;
out vec4 f_color;

const float pi = 3.141593;
const float inc_rate = 0.01;
const float range = 100;
const int max_lights = 64;
const float lightmap_stretch = 2.0;
const float expanded_uv_min = -(lightmap_stretch - 1) * 0.5;

uniform vec3 lights[max_lights];
uniform int light_count;

float sample_map(vec2 uv) {
  // strecth at 2.0 makes bounds -0.5 to 1.5 instead of 0.0 to 1.0
  // coordinates need to be condensed to 0.0 to 1.0 range
  uv = vec2((uv.x - expanded_uv_min) / lightmap_stretch, (uv.y - expanded_uv_min) / lightmap_stretch);
  return texture(map, uv).r;
}

void main() {
  f_color = vec4(0.7);

  int light_i = int((vert.y + 1) / 2 * max_lights);
  vec3 light;
  if (light_i < 0) {
    light = lights[0];
  } else {
    light = lights[light_i];
  }

  float angle = vert.x * pi;
  vec2 inc = vec2(cos(angle) * inc_rate, sin(angle) * inc_rate);
  float dis = 0.0;
  for (int i = 0; i < range; i++) {
    vec2 sample_pos = vec2(light.x + inc.x * i, light.y + inc.y * i);
    float map_c = step(0.01, sample_map(sample_pos));
    if (dis == 0) {
      dis = map_c * (range - i) * inc_rate;
    }
  }

  gl_FragDepth = dis;
  f_color = vec4(dis);
}
