#version 330

uniform sampler2D shadowmap;
uniform sampler2D glow;
//uniform sampler2D tex;

in vec2 uv;
out vec4 f_color;

const float pi = 3.141593;
const float inv_pi_over_2 = 1.0 / (pi * 2.0);
const float inc_rate = 0.01;
const float range = 100;
const int max_lights = 64;
const float impact_falloff = 0.1;
const float inv_impact_falloff = 1 / impact_falloff;
const int sample_normalize = 10;
const float ray_sample_width = 1.0 / 640.0;

uniform vec3 lights[max_lights];
uniform vec3 light_colors[max_lights];
uniform vec2 light_ranges[max_lights];
uniform int light_count;
uniform float aspect_ratio;

float shadow_dis_avg(float angle, int light_index) {
  float total = 0;
  for (int i = 0; i < sample_normalize; i++) {
    total += texture(shadowmap, vec2(angle + i * ray_sample_width, (float(light_index) + 0.5) / max_lights)).r;
    total += texture(shadowmap, vec2(angle - i * ray_sample_width, (float(light_index) + 0.5) / max_lights)).r;
  }
  return total / (sample_normalize * 2);
}

void main() {
  vec3 visibility = vec3(0.0);

  // make a grayscale image instead of blending with texture
  vec3 tex_color = vec3(1.0, 1.0, 1.0);
  //vec3 tex_color = texture(tex, uv).rgb + 0.2;

  for (int i = 0; i < max_lights; i++) {
    vec3 light = lights[i];
    vec2 light_range = light_ranges[i];
    vec2 vdis = vec2(uv.x - light.x, uv.y - light.y);
    float angle = atan(vdis.y, vdis.x);
    float dis = sqrt(vdis.x * vdis.x + vdis.y * vdis.y);
    float circular_falloff_dis = sqrt(vdis.x * vdis.x + pow(vdis.y * aspect_ratio, 2));
    float shadow_lookup = (angle * inv_pi_over_2) + 0.5;
    //float shadow_dis = shadow_dis_avg(shadow_lookup, i);
    float shadow_dis = texture(shadowmap, vec2(shadow_lookup, (float(i) + 0.5) / max_lights)).r;

    float light_scale = light.z * 0.25;
    if (mod((angle - light_range[0]), pi * 2) < light_range[1]) {
      light_scale *= 4.0;
    }
    if (((1.0 - shadow_dis) > dis - impact_falloff) && (i < light_count)) {
      //float natural_falloff = pow(max(0.0, (light.z * 0.6 - circular_falloff_dis) / (light.z * 0.6)), 2) * 0.7 + sqrt(max(0.0, (light.z - circular_falloff_dis) / light.z)) * 0.3;
      float natural_falloff = pow(max(0.0, (light_scale * 0.15 - circular_falloff_dis) / (light_scale * 0.15)), 2) * 0.8 + pow(max(0.0, ((light_scale * 0.9) - circular_falloff_dis) / (light_scale * 0.9)), 2) * 0.8;
      visibility += min(1.0, ((1.0 - shadow_dis) - (dis - impact_falloff)) * inv_impact_falloff) * light_colors[i] * natural_falloff;
    }
  }
  tex_color *= visibility + texture(glow, uv).rgb;
  //tex_color.r += texture(shadowmap, uv).r * 0.15;
  f_color = vec4(tex_color, 1.0);
}
